/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 */

#ifndef MUTT_LIB_LIB_HASH_H
#define MUTT_LIB_LIB_HASH_H

struct hash_elem {
    const char *key;
    void *data;
    struct hash_elem *next;
};

typedef struct {
    unsigned dupes : 1;
    int nelem;
    int curnelem;
    struct hash_elem **table;
} hash_t;

hash_t *hash_init(hash_t *, int nelem, int allow_dup);
static inline hash_t *hash_new(int nelem, int allow_dup) {
    return hash_init(p_new(hash_t, 1), nelem, allow_dup);
}

void hash_wipe(hash_t*, void (*dtor)(void*));
static inline void hash_delete(hash_t **h, void (*dtor)(void*)) {
    if (*h) {
        hash_wipe(*h, dtor);
        p_delete(h);
    }
}

int hash_string(const unsigned char *s, int n);
void hash_resize(hash_t *table, int nelem);

int hash_insert(hash_t *table, const char *key, void *data);
void *hash_find(const hash_t *table, const char *key);
void hash_remove(hash_t *table, const char *key, const void *data,
                 void (*destroy)(void *));

void hash_map(hash_t *table,
              void (*mapfunc)(const char *key, void *data, unsigned long more),
              unsigned long more);

#endif /* MUTT_LIB_LIB_HASH_H */
